package main

import (
	"flag"
	"fmt"
	"log"
	"net/netip"
	"os"
	"os/signal"
	"strings"
	"syscall"
	"time"

	"git.arav.top/Arav/dwelling-home/internal/http"
)

var version string

var showVersion *bool = flag.Bool("v", false, "show version")
var listenAddress *string = flag.String("listen", "/var/run/dwelling-home/sock", "listen address (ip:port|unix_path)")
var captchaExpiry *time.Duration = flag.Duration("captcha-expiry", 10*time.Minute, "CAPTCHA expiry (e.g. 5m, 60s)")
var guestbookOwner *string = flag.String("gb-owner", "Admin", "name of a guestbook owner")
var guestbookPageSize *int = flag.Int("gb-pagesz", 60, "size of a guestbook page")
var databasesPath *string = flag.String("db", "/var/lib/dwelling-home", "path to a directory where to store DB files")

func main() {
	flag.Parse()

	if *showVersion {
		fmt.Println("dwelling-home Ver.", version, "\nCopyright (c) 2023 Alexander \"Arav\" Andreev <me@arav.top>")
		return
	}

	var network string
	if !strings.ContainsRune(*listenAddress, ':') {
		network = "unix"
		defer os.Remove(*listenAddress)
	} else {
		ap, err := netip.ParseAddrPort(*listenAddress)
		if err != nil {
			log.Fatalln(err)
		}

		if ap.Addr().Is4() {
			network = "tcp4"
		} else if ap.Addr().Is6() {
			network = "tcp6"
		}
	}

	hand := http.NewHandlers(*captchaExpiry, *databasesPath, *guestbookOwner, *guestbookPageSize)
	srv := http.NewHttpServer()

	srv.ServeStatic("/assets/*filepath", hand.AssetsFS())
	srv.GET("/", hand.Index)
	srv.GET("/robots.txt", hand.Robots)

	if err := srv.Start(network, *listenAddress); err != nil {
		log.Fatalln(err)
	}

	doneSignal := make(chan os.Signal, 1)
	signal.Notify(doneSignal, os.Interrupt, syscall.SIGINT, syscall.SIGTERM)

	<-doneSignal

	if err := srv.Stop(); err != nil {
		log.Fatalln(err)
	}
}
